import matplotlib.pyplot as plt
import numpy as np
import pandas
pandas.set_option('display.max_columns', None)
import uproot
import os


##############################################################
# H I T S 
##############################################################

# Load the hits registered by the crystalSD detector:
rootfile  = uproot.open("./Simulation/output/0.hits.root")
crystalSD = rootfile['tree'].arrays(filter_name=["*"], library='pd')

# For each particle, extract only the first interaction
# with the crystalSD; this is done by checking when 
# the eventID changes in the array.

# First, a boolean mask array is created...
temp = (crystalSD['eventID'][1:].to_numpy() - crystalSD['eventID'][:-1].to_numpy())>0
onlyFirstHitBoolean = np.zeros(temp.size + 1, dtype=bool)
onlyFirstHitBoolean[0] = 1
onlyFirstHitBoolean[1:] = temp

# ...and then applied:
crystalSD_onlyFirstHit = crystalSD.loc[onlyFirstHitBoolean,:]

assert crystalSD_onlyFirstHit.shape[0] == len(set(crystalSD['eventID']))

# Determine the number of photons hitting the detector:
numberOfIncidentPhotons = len(set(crystalSD['eventID']))

# Extract the number of events whose flags indicate that the 
# photon has undergone Compton or Rayleigh interaction or which 
# is a secondary:
numberOfScatterAndSecondaries = np.sum( (crystalSD_onlyFirstHit['nPhantomCompton']>0) | (crystalSD_onlyFirstHit['nPhantomRayleigh']>0) | (crystalSD_onlyFirstHit['parentID']>0))

# Extract the number of events which hit the detector off-centre:
numberOfHitsOffCentre = crystalSD_onlyFirstHit[ (np.abs(crystalSD_onlyFirstHit['posX']) > 1E-4) | (np.abs(crystalSD_onlyFirstHit['posY']) > 1E-4) ].shape[0]




##############################################################
# P H A S E   S P A C E   A C T O R
##############################################################

# Load the result of the phase space actor in front of the 
# detector:
rootfile  = uproot.open("./Simulation/output/detector.root")
phs = rootfile['PhaseSpace'].arrays(filter_name="*", library='pd')

# Extract only those photons that travel from the direction of 
# the source into the direciton of the detector; this way, 
# photons scattered in the detector and passing back through 
# the phase space actor in front of it are omitted:
phs_posDir = phs[phs['dZ'] < 0]

# Count the number of events that pass through the phase space 
# actor off-centre:
numberPhaseSpaceOPffCentre = np.sum( ((np.abs(phs_posDir['X'])>1E-4) | (np.abs(phs_posDir['Y'])>1E-4)) )


##############################################################
# P R I N T   R E S U L T S
##############################################################
print("Number of photons incident on the detector:                          {:>6}".format(numberOfIncidentPhotons))
print("Number of scattered particles and secondaries as registered by Gate: {:>6}".format(numberOfScatterAndSecondaries))
print("Number of registered hits off-centre:                                {:>6}".format(numberOfHitsOffCentre))
print("Number of events registered by the phase space actor off-centre:     {:>6}".format(numberPhaseSpaceOPffCentre))





##############################################################
# PLOT RESULTS
##############################################################

# The following sections hold the results obtained for 
# setting either a range cut for the detector pixels only
# or for the water phantom only, and plots them.


#-----------------------#
# Range cut in detector
#-----------------------#

#Number of photons incident on the detector: 
numberOfIncidentPhotonsArray_DetectorRC = [13709, 13709, 13709, 13599, 13578, 13620]

# Number of scattered particles and secondaries as registered by Gate: 
numberOfScatterAndSecondariesArray_DetectorRC = [9733, 9733, 9733, 2398, 2217, 2174]

# Number of registered hits off-centre:
numberOfHitsOffCentreArray_DetectorRC = [9730, 9730, 9730, 9697, 9722, 9668]

# Number of events registered by the phase space actor off-centre: 
numberPhaseSpaceOPffCentre_DetectorRC = [9788, 9788, 9788, 9750, 9773, 9733]


### Plot

fig, ax = plt.subplots()

x = np.arange(len(numberOfIncidentPhotonsArray_DetectorRC))

ax.plot(x, numberOfIncidentPhotonsArray_DetectorRC, marker='*', label='incident photons')
ax.plot(x, numberOfScatterAndSecondariesArray_DetectorRC, marker='o', label='Gate: scatter & secondaries')
ax.plot(x, numberOfHitsOffCentreArray_DetectorRC, marker='s', label='hits off-centre')
ax.plot(x, numberPhaseSpaceOPffCentre_DetectorRC, marker='d', label='off-centre in phase-space')

ax.set_xlabel("Range cut [mm]")
ax.set_ylabel("Number of events")

ax.legend(loc='upper left', bbox_to_anchor=(1.0, 0.5))

plt.xticks(x, rangeCutArray)

# ax.set_ylim(13000, 14000)

# plt.savefig("temp.png", dpi=300, bbox_inches='tight')

plt.show()



#-----------------------#
# Range cut in phantom
#-----------------------#

# Number of photons incident on the detector: 
numberOfIncidentPhotonsArray_PhantomRC = [13709, 13709, 13709, 13550, 13503, 13568]

# Number of scattered particles and secondaries as registered by Gate: 
numberOfScatterAndSecondariesArray_PhantomRC = [9733, 9733, 9733, 9689, 9426, 9482]

# Number of registered hits off-centre:  
numberOfHitsOffCentreArray_PhantomRC = [9730, 9730, 9730, 9681, 9446, 9523]

# Number of events registered by the phase space actor off-centre:  
numberPhaseSpaceOPffCentre_PhantomRC = [9788, 9788, 9788, 9742, 9504, 9595]

rangeCutArray = ['None', '10', '1', '0.1', '0.01', '0.001']    


### Plot
fig, ax = plt.subplots()

x = np.arange(len(numberOfIncidentPhotonsArray_PhantomRC))

ax.plot(x, numberOfIncidentPhotonsArray_PhantomRC, marker='*', label='incident photons')
ax.plot(x, numberOfScatterAndSecondariesArray_PhantomRC, marker='o', label='Gate: scatter & secondaries')
ax.plot(x, numberOfHitsOffCentreArray_PhantomRC, marker='s', label='hits off-centre')
ax.plot(x, numberPhaseSpaceOPffCentre_PhantomRC, marker='d', label='off-centre in phase-space')

ax.set_xlabel("Range cut [mm]")
ax.set_ylabel("Number of events")

ax.legend(loc='upper left', bbox_to_anchor=(1.0, 0.5))

plt.xticks(x, rangeCutArray)

# ax.set_ylim(9000, 10000)

# plt.savefig("temp.png", dpi=300, bbox_inches='tight')

plt.show()